/*
 * This file is part of be_cbo.
 * Copyright 2009 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#include "Application.h"
#include "Convert.h"
#include "InputHandler.h"
#include "GeodeData.h"
#include "TransformData.h"

#include <osg/PositionAttitudeTransform>
#include <osg/ShapeDrawable>


#include <osgDB/ReadFile>

#include <osgGA/TrackballManipulator>

#include <osgUtil/IntersectionVisitor>
#include <osgUtil/LineSegmentIntersector>

#include <osgViewer/ViewerEventHandlers>

#include <cstdlib>
#include <ctime>

const osg::ref_ptr<osg::Texture2D> Application::g_woodTexture = new osg::Texture2D(osgDB::readImageFile("data/Wood.png"));
const osg::ref_ptr<osg::Texture2D> Application::g_graniteTexture = new osg::Texture2D(osgDB::readImageFile("data/Granite.png"));

const float Application::g_woodDensity = 500.0f;
const float Application::g_graniteDensity = 2800.0f;

Application::Application() : m_updatePhysics(true) {
	initBullet();
	initOpenSceneGraph();

	addGroundPlane();
	addRandomBoxes();
}

Application::~Application() {
	destroyOpenSceneGraph();
	destroyBullet();
}

void Application::mainLoop() {
	double lastTime = 0.0;

	while(!m_viewer.done()) {
		// Get the time difference between the current and the last frame.
		double simTime = m_viewer.getFrameStamp()->getSimulationTime();
		double diffTime = simTime - lastTime;
		lastTime = simTime;

		// Calculate the physics.
		if(m_updatePhysics) {
			m_world->stepSimulation(diffTime);
		}

		// Display the frame.
		m_viewer.frame();
	}
}

void Application::addPhysicsNode(	osg::Texture * texture,
						Shape * shape,
						btVector3 center,
						btScalar mass,
						btVector3 velocity) {
	osg::PositionAttitudeTransform * transform = new osg::PositionAttitudeTransform;
	TransformData * transformData = new TransformData(transform);
	transform->setUserData(transformData);
	transform->setPosition(Convert::fromBtVec(center));

	btVector3 localInertia(0, 0, 0);
	shape->getCollisionShape()->calculateLocalInertia(mass, localInertia);
	btRigidBody::btRigidBodyConstructionInfo constructionInfo(mass, transformData, shape->getCollisionShape(), localInertia);
	
	osg::Geode * geode = new osg::Geode;
	GeodeData * geodeData = new GeodeData(m_world, constructionInfo, shape);
	geode->setUserData(geodeData);
	geode->addDrawable(new osg::ShapeDrawable(shape->getOsgShape()));
	geode->getOrCreateStateSet()->setTextureAttributeAndModes(0, texture);

	if(velocity.length() > 0.0f) {
		geodeData->getRigidBody()->setLinearVelocity(velocity);
	}

	dynamic_cast<osg::Group *>(m_viewer.getSceneData())->addChild(transform);
	transform->addChild(geode);
}

void Application::addGroundPlane() {
	addPhysicsNode(g_graniteTexture.get(), new Box(btVector3(150.0f, 1.0f, 150.0f)), btVector3(0.0f, 0.0f, 0.0f), 0.0f);
}

void Application::addRandomBoxes() {
	// (Re-)Initialize the random number generator.
	srand(time(NULL));

	const float randomMax = static_cast<float>(RAND_MAX);
	btVector3 pos;
	btVector3 scale;
	btScalar mass;
	for(unsigned int i = 0; i < 200; ++i) {
		pos.setX(-50.0f + 100.0f * (rand() / randomMax));
		pos.setY(  50.0f + 100.0f * (rand() / randomMax));
		pos.setZ(-50.0f + 100.0f * (rand() / randomMax));
		scale.setX(0.5f + 4.5f * (rand() / randomMax));
		scale.setY(0.5f + 4.5f * (rand() / randomMax));
		scale.setZ(0.5f + 4.5f * (rand() / randomMax));
		
		mass = scale.x() * scale.y() * scale.z() * g_woodDensity;
		addPhysicsNode(g_woodTexture.get(), new Box(scale), pos, mass);
	}
}

void Application::togglePhysicsUpdate() {
	m_updatePhysics = !m_updatePhysics;
}

void Application::initBullet() {
	m_collisionConfiguration = new btDefaultCollisionConfiguration;
	m_dispatcher = new btCollisionDispatcher(m_collisionConfiguration);
	m_broadphase = new btDbvtBroadphase;
	m_solver = new btSequentialImpulseConstraintSolver;
	m_world = new btDiscreteDynamicsWorld(m_dispatcher, m_broadphase, m_solver, m_collisionConfiguration);
	m_world->setGravity(btVector3(0.0f, -9.81f, 0.0f));
}

void Application::destroyBullet() {
	delete m_world;
	delete m_solver;
	delete m_broadphase;
	delete m_dispatcher;
	delete m_collisionConfiguration;
}

void Application::initOpenSceneGraph() {
	// Create a view into the scene.
	m_viewer.setSceneData(new osg::Group);

	// Statistics can be activated with key 's'.
	m_viewer.addEventHandler(new osgViewer::StatsHandler());
	
	// Allow changing the camera with mouse and keyboard.
	m_viewer.setCameraManipulator(new osgGA::TrackballManipulator);
	m_viewer.getCameraManipulator()->setHomePosition(osg::Vec3(0.0f, 15.0f, -150.0f), osg::Vec3(0.0f, 15.0f, 0.0f), osg::Vec3(0.0f, 1.0f, 0.0f));

	// Allow selection of objects in the 3D view.
	m_viewer.addEventHandler(new InputHandler(*this));

	// Initialize the viewer and let it create the rendering context.
	m_viewer.realize();
}

void Application::destroyOpenSceneGraph() {
	m_viewer.setSceneData(NULL);
}
