/*
 * This file is part of be_cbo.
 * Copyright 2009 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#include "InputHandler.h"
#include "Application.h"
#include "Convert.h"
#include "DeleteSelectedNodesVisitor.h"

InputHandler::InputHandler(Application & app) : 
		m_application(app),
		m_mouseX(0.0f),
		m_mouseY(0.0f) {
}

bool InputHandler::handle(const osgGA::GUIEventAdapter & ea, osgGA::GUIActionAdapter & aa) {
	osgViewer::Viewer * viewer = dynamic_cast<osgViewer::Viewer *>(&aa);
	if(viewer == NULL) {
		return false;
	}

	osg::Group * rootNode = dynamic_cast<osg::Group *>(viewer->getSceneData());
	if(rootNode == NULL) {
		return false;
	}

	switch(ea.getEventType()) {
		case(osgGA::GUIEventAdapter::KEYUP): 
			if(	ea.getKey() == '1' ||
				ea.getKey() == '2' ||
				ea.getKey() == '3') {
				osg::Vec3 eye;
				osg::Vec3 center;
				osg::Vec3 up;
				viewer->getCamera()->getViewMatrixAsLookAt(eye, center, up);
				osg::Vec3 dir = (center - eye) * 50.0f;
				btScalar radius;
				if(ea.getKey() == '1') {
					radius = 0.4f;
				} else if(ea.getKey() == '2') {
					radius = 1.5f;
				} else {
					radius = 3.0f;
				}  
				// 4.18879020479 = PI * 4 / 3
				btScalar mass = 4.18879020479f * radius * radius * radius * Application::g_graniteDensity;
				
				m_application.addPhysicsNode(Application::g_graniteTexture.get(), new Sphere(radius), Convert::toBtVec(eye), mass, Convert::toBtVec(dir));
			} else if(ea.getKey() == 'r') {
				osg::Group * rootNode = dynamic_cast<osg::Group *>(viewer->getSceneData());
				rootNode->removeChild(0, rootNode->getNumChildren());
				m_application.addGroundPlane();
				m_application.addRandomBoxes();
			} else if(ea.getKey() == 'n') {
				m_application.addRandomBoxes();
			} else if(ea.getKey() == osgGA::GUIEventAdapter::KEY_Delete || ea.getKey() == osgGA::GUIEventAdapter::KEY_BackSpace) {
				DeleteSelectedNodesVisitor dsnv;
				viewer->getSceneData()->accept(dsnv);
				dsnv.pruneSelectedNodes();
			} else if(ea.getKey() == 'p') {
				m_application.togglePhysicsUpdate();
			}
			return false;
		case(osgGA::GUIEventAdapter::PUSH):
		case(osgGA::GUIEventAdapter::MOVE):
			m_mouseX = ea.getX();
			m_mouseY = ea.getY();
			return false;
		case(osgGA::GUIEventAdapter::RELEASE):
			if (m_mouseX == ea.getX() && m_mouseY == ea.getY()) {
				// only do a pick if the mouse hasn't moved
				pick(ea, viewer);
			}
			return true;
		default:
			return false;
	}
}

void InputHandler::pick(const osgGA::GUIEventAdapter & ea, osgViewer::Viewer * viewer) {
	osgUtil::LineSegmentIntersector * picker = new osgUtil::LineSegmentIntersector(osgUtil::Intersector::PROJECTION, ea.getXnormalized(), ea.getYnormalized());

	osgUtil::IntersectionVisitor iv(picker);

	viewer->getCamera()->accept(iv);

	if (picker->containsIntersections()) {
		osgUtil::LineSegmentIntersector::Intersection intersection = picker->getFirstIntersection();

		osg::NodePath & nodePath = intersection.nodePath;
		osg::Node * node = (nodePath.size() >= 1) ? nodePath[nodePath.size() - 1] : NULL;
		osg::Group * parent = (nodePath.size() >= 2) ? dynamic_cast<osg::Group *>(nodePath[nodePath.size() - 2]) : NULL;

		toggleScribe(parent, node);
	}
}

void InputHandler::toggleScribe(osg::Group * parent, osg::Node * node) {
	if(parent == NULL || node == NULL) {
		return;
	}

	osgFX::Scribe * parentAsScribe = dynamic_cast<osgFX::Scribe *>(parent);
	if(parentAsScribe == NULL) {
		// node not already picked, so highlight it with an osgFX::Scribe
		osgFX::Scribe * scribe = new osgFX::Scribe;
		scribe->addChild(node);
		parent->replaceChild(node, scribe);
	} else {
		// node already picked so we want to remove scribe to unpick it.
		osg::Node::ParentList parentList = parentAsScribe->getParents();
		for(osg::Node::ParentList::iterator itr = parentList.begin(); itr != parentList.end(); ++itr) {
			(*itr)->replaceChild(parentAsScribe, node);
		}
	}
}
