/*
 * This file is part of be_cbo.
 * Copyright 2009 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef SHAPE_H
#define SHAPE_H

#include <BulletCollision/CollisionShapes/btCollisionShape.h>
#include <BulletCollision/CollisionShapes/btBoxShape.h>
#include <BulletCollision/CollisionShapes/btSphereShape.h>
#include <osg/Shape>

/**
 * Class representing one abstract collision object. It contains pointers to a 
 * btCollisionShape and and osg::Shape. This class is intended to be 
 * subclassed.
 *
 * @brief %Shape of a collision object
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class Shape {
	public:
		/**
		 * Store the OpenSceneGraph and Bullet shapes.
		 *
		 * @param collisionShape Bullet shape
		 * @param osgShape OpenSceneGraph shape
		 */
		Shape(btCollisionShape * collisionShape, osg::Shape * osgShape) : m_collisionShape(collisionShape), m_osgShape(osgShape) {
		}

		/**
		 * Free the stored objects.
		 */
		~Shape() {
			delete m_collisionShape;
			m_osgShape = NULL;
		}

		/**
		 * Access to the Bullet shape.
		 *
		 * @return Bullet shape
		 */
		btCollisionShape * getCollisionShape() {
			return m_collisionShape;
		}

		/**
		 * Constant access to the Bullet shape.
		 *
		 * @return Bullet shape
		 */
		const btCollisionShape * getCollisionShape() const {
			return m_collisionShape;
		}

		/**
		 * Access to the OpenSceneGraph shape.
		 *
		 * @return OpenSceneGraph shape
		 */
		osg::Shape * getOsgShape() {
			return m_osgShape;
		}

		/**
		 * Constant access to the OpenSceneGraph shape.
		 *
		 * @return OpenSceneGraph shape
		 */
		const osg::Shape * getOsgShape() const {
			return m_osgShape;
		}

	private:
		/**
		 * Pointer to the Bullet shape.
		 */
		btCollisionShape * m_collisionShape;

		/**
		 * Pointer to the OpenSceneGraph shape.
		 */
		osg::ref_ptr<osg::Shape> m_osgShape;
};

/**
 * Special shape reprensenting a box.
 *
 * @brief %Box shape
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class Box : public Shape {
	public:
		/**
		 * Create a sphere with the given side lengths.
		 *
		 * @param size Side lengths of the box.
		 */
		Box(btVector3 size) : Shape(new btBoxShape(0.5 * size), new osg::Box(osg::Vec3(0.0f, 0.0f, 0.0f), size.x(), size.y(), size.z())) {
		}
};

/**
 * Special shape reprensenting a sphere.
 *
 * @brief %Sphere shape
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class Sphere : public Shape {
	public:
		/**
		 * Create a sphere with the given radius.
		 *
		 * @param radius Radius of the sphere.
		 */
		Sphere(btScalar radius) : Shape(new btSphereShape(radius), new osg::Sphere(osg::Vec3(0.0f, 0.0f, 0.0f), radius)) {
		}
};

#endif // SHAPE_H
