/*
 * This file is part of be_cbo.
 * Copyright 2009 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef TRANSFORMDATA_H
#define TRANSFORMDATA_H

#include "Convert.h"
#include <LinearMath/btMotionState.h>
#include <osg/PositionAttitudeTransform>

/**
 * Class for linking an osg::PositionAttitudeTransform with a btTransform. The 
 * position and attitude will be synchronized between OpenSceneGraph and 
 * Bullet. Intented to be set as user data of an
 * osg::PositionAttitudeTransform.
 *
 * @brief Synchronization of transformations.
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class TransformData : public osg::Referenced, public btMotionState {
	public:
		/**
		 * Create the data object with a pointer to the transformation node 
		 * that is going to be updated.
		 *
		 * @param transfrom Transformation node
		 */
		TransformData(osg::PositionAttitudeTransform * transform) : m_transform(transform) {
		}

		/**
		 * Do nothing.
		 */
		virtual ~TransformData() {
		}

		/**
		 * Return the transformation of the OpenSceneGraph node. This is read 
		 * by the physics library.
		 *
		 * @param [out] worldTrans Return the current transformation.
		 */
		virtual void getWorldTransform(btTransform & worldTrans) const {
			worldTrans.setOrigin(Convert::toBtVec(m_transform->getPosition()));
			worldTrans.setRotation(Convert::toBtQuat(m_transform->getAttitude()));
		}

		/**
		 * Set the transformation of the OpenSceneGraph node. This is written 
		 * by the physics library when something has changed.
		 *
		 * @param [in] worldTrans Set the current transformation.
		 */
		virtual void setWorldTransform(const btTransform & worldTrans) {
			m_transform->setPosition(Convert::fromBtVec(worldTrans.getOrigin()));
			m_transform->setAttitude(Convert::fromBtQuat(worldTrans.getRotation()));
		}

	protected:
		/**
		 * Pointer to the transformation node which will be synchronized with
		 * the physics calculations.
		 */
		osg::PositionAttitudeTransform * m_transform;
};

#endif // TRANSFORMDATA_H
